@@ -28,14 +28,15 @@
 #include "ARM9.h"
 #include <stdio.h>
 #include "mem.h"
+#include "common.h"
 #include "registers.h"
 #include "FIFO.h"
 #include "MMU.h"
+#include "GPU_osd.h"
+#include <iosfwd>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
+void gpu_savestate(std::ostream* os);
+bool gpu_loadstate(std::istream* is);
 
 /*******************************************************************************
     this structure is for display control,
@@ -48,7 +49,7 @@ struct _DISPCNT
 /* 7*/  u8 ForceBlank:1;      // A+B:
 /* 6*/  u8 OBJ_BMP_mapping:1; // A+B: 0=2D (128KB), 1=1D (128..256KB)
 /* 5*/  u8 OBJ_BMP_2D_dim:1;  // A+B: 0=128x512,    1=256x256 pixels
-/* 4*/  u8 OBJ_Tile_1D:1;     // A+B: 0=2D (32KB),  1=1D (32..256KB)
+/* 4*/  u8 OBJ_Tile_mapping:1;// A+B: 0=2D (32KB),  1=1D (32..256KB)
 /* 3*/  u8 BG0_3D:1;          // A  : 0=2D,         1=3D
 /* 0*/  u8 BG_Mode:3;         // A+B:
 /*15*/  u8 WinOBJ_Enable:1;   // A+B: 0=disable, 1=Enable
@@ -80,7 +81,7 @@ struct _DISPCNT
 {
 /* 0*/  u8 BG_Mode:3;         // A+B:
 /* 3*/  u8 BG0_3D:1;          // A  : 0=2D,         1=3D
-/* 4*/  u8 OBJ_Tile_1D:1;     // A+B: 0=2D (32KB),  1=1D (32..256KB)
+/* 4*/  u8 OBJ_Tile_mapping:1;     // A+B: 0=2D (32KB),  1=1D (32..256KB)
 /* 5*/  u8 OBJ_BMP_2D_dim:1;  // A+B: 0=128x512,    1=256x256 pixels
 /* 6*/  u8 OBJ_BMP_mapping:1; // A+B: 0=2D (128KB), 1=1D (128..256KB)
 
@@ -341,32 +342,25 @@ typedef union
     source:
     http://nocash.emubase.de/gbatek.htm#dsvideocaptureandmainmemorydisplaymode
 *******************************************************************************/
-
-struct _DISPCAPCNT
+typedef struct
 {
-/* 0*/ u8 BlendFactor_A:5;     // 0..16 = Blending Factor for Source A
-/* 5*/ u8 :3;                  //
-/* 8*/ u8 BlendFactor_B:5;     // 0..16 = Blending Factor for Source B
-/*13*/ u8 :3;                  //
-/*16*/ u8 VRAM_Write_Block:2;  // 0..3 = VRAM A..D
-/*18*/ u8 VRAM_Write_Offset:2; // n x 0x08000
-/*20*/ u8 Capture_Size:2;      // 0=128x128, 1=256x64, 2=256x128, 3=256x192 dots
-/*22*/ u8 :2;                  //
-/*24*/ u8 Source_A:1;          // 0=Graphics Screen BG+3D+OBJ, 1=3D Screen
-/*25*/ u8 Source_B:1;          // 0=VRAM, 1=Main Memory Display FIFO
-/*26*/ u8 VRAM_Read_Offset:2;  // n x 0x08000
-/*28*/ u8 :1;                  //
-/*29*/ u8 Capture_Source:2;    // 0=Source A, 1=Source B, 2/3=Sources A+B blended
-/*31*/ u8 Capture_Enable:1;    // 0=Disable/Ready, 1=Enable/Busy
-};
-
-typedef union
-{
-    struct _DISPCAPCNT bits;
-    u32 val;
+    u32		val;
+	BOOL	enabled;
+	u8		EVA;
+	u8		EVB;
+	u8		writeBlock;
+	u8		writeOffset;
+	u16		capx;
+	u16		capy;
+	u8		srcA;
+	u8		srcB;
+	u8		readBlock;
+	u8		readOffset;
+	u8		capSrc;
+	u8		*dst;
+	u8		*src;
 } DISPCAPCNT;
 
-
 /*******************************************************************************
     this structure holds everything and should be mapped to
     * core A : 0x04000000
@@ -389,20 +383,6 @@ typedef struct _reg_dispx {
 } REG_DISPx ;
 
 
-
-
-
-
-
-
-#ifndef min
-#define min(a,b) (((a)<(b))?(a):(b))
-#endif
-
-#ifndef max
-#define max(a,b) (((a)>(b))?(a):(b))
-#endif
-
 typedef BOOL (*fun_gl_Begin) (int screen);
 typedef void (*fun_gl_End) (int screen);
 // the GUI should use this function prior to all gl calls
@@ -466,10 +446,17 @@ typedef union
 */
 
 struct _COLOR { // abgr x555
-	unsigned red:5;
-	unsigned green:5;
+#ifdef WORDS_BIGENDIAN
+	unsigned alpha:1;    // sometimes it is unused (pad)
 	unsigned blue:5;
-	unsigned alpha:1;	// sometimes it is unused (pad)
+	unsigned green:5;
+	unsigned red:5;
+#else
+     unsigned red:5;
+     unsigned green:5;
+     unsigned blue:5;
+     unsigned alpha:1;    // sometimes it is unused (pad)
+#endif
 };
 struct _COLORx { // abgr x555
 	unsigned bgr:15;
@@ -521,6 +508,27 @@ ref: http://www.bottledlight.com/ds/index.php/Video/Sprites
 
 typedef struct
 {
+#ifdef WORDS_BIGENDIAN
+// attr0
+/* 0*/    unsigned Y:8;
+/*14*/    unsigned Shape:2;    // (00: Square, 01: Wide, 10: Tall, 11: Illegal)
+/*13*/    unsigned Depth:1;    // (0: 16, 1: 256)
+/*12*/    unsigned Mosaic:1;   // (1: Enabled)
+/*10*/    unsigned Mode:2;     // (00: Normal, 01: Transparent, 10: Object window, 11: Bitmap)
+/* 8*/    unsigned RotScale:2; // (00: Normal, 01: Rot/scale, 10: Disabled, 11: Double-size rot/scale)
+// attr1
+/* 0*/    signed   X:9;
+/*14*/    unsigned Size:2;
+/*13*/    unsigned VFlip:1;
+/*12*/    unsigned HFlip:1;
+/* 9*/    unsigned RotScalIndex:3; // Rot/scale matrix index
+// attr2
+/* 0*/    unsigned TileIndex:10;
+/*12*/    unsigned PaletteIndex:4;
+/*10*/    unsigned Priority:2;
+// attr3
+unsigned attr3:16;    
+#else
 // attr0
 /* 0*/	unsigned Y:8;
 /* 8*/	unsigned RotScale:2; // (00: Normal, 01: Rot/scale, 10: Disabled, 11: Double-size rot/scale)
@@ -540,6 +548,7 @@ typedef struct
 /*12*/	unsigned PaletteIndex:4;
 // attr3
 	unsigned attr3:16;
+#endif
 } _OAM_;
 
 typedef struct
@@ -575,11 +584,14 @@ typedef struct
 	// doh ! yoda says : 256 pixels we can have...
 	u16 nbPixelsX;
 } itemsForPriority_t;
+#define ARM9MEM_ABG		0x06000000
+#define ARM9MEM_BBG		0x06200000
+#define ARM9MEM_AOBJ	0x06400000
+#define ARM9MEM_BOBJ	0x06600000
 
+extern CACHE_ALIGN u8 gpuBlendTable555[17][17][32][32];
 
-typedef struct _GPU GPU;
-
-struct _GPU
+struct GPU
 {
 	// some structs are becoming redundant
 	// some functions too (no need to recopy some vars as it is done by MMU)
@@ -588,29 +600,50 @@ struct _GPU
 	DISPCAPCNT dispCapCnt;
 	BOOL LayersEnable[5];
 	itemsForPriority_t itemsForPriority[NB_PRIORITIES];
-	u8 sprWin[256][256];
 
 #define BGBmpBB BG_bmp_ram
 #define BGChBB BG_tile_ram
 
-	u8 *(BG_bmp_ram[4]);
-	u8 *(BG_tile_ram[4]);
-	u8 *(BG_map_ram[4]);
+	u32 BG_bmp_ram[4];
+	u32 BG_tile_ram[4];
+	u32 BG_map_ram[4];
 
 	u8 BGExtPalSlot[4];
 	u32 BGSize[4][2];
 
-	u8 lcd;
+	struct MosaicColor {
+		u16 bg[4][256];
+		struct Obj {
+			u16 color;
+			u8 alpha, opaque;
+		} obj[256];
+	} mosaicColors;
+
+	u8 sprNum[256];
+	u8 h_win[2][256];
+	const u8 *curr_win[2];
+	void update_winh(int WIN_NUM); 
+	bool need_update_winh[2];
+	
+	template<int WIN_NUM> void setup_windows();
+
 	u8 core;
 
 	u8 dispMode;
 	u8 vramBlock;
+	u8 *VRAMaddr;
+
+	//FIFO	fifo;
 
 	BOOL dispBG[4];
 	BOOL dispOBJ;
 
+	u8 bgPrio[5];
+
+	BOOL bg0HasHighestPrio;
+
 	OAM * oam;
-	u8 * sprMem;
+	u32	sprMem;
 	u8 sprBoundary;
 	u8 sprBMPBoundary;
 	u8 sprBMPMode;
@@ -648,8 +681,98 @@ struct _GPU
 	u8	MasterBrightMode;
 	u32 MasterBrightFactor;
 
-	BOOL (*setFinalColor)(const GPU *gpu, u32 passing, u8 bgnum, u8 *dst, u16 color, u16 x, u16 y);
-	void (*spriteRender) (GPU * gpu, u16 l, u8 * dst, u8 * prioTab);
+	u8 bgPixels[256];
+
+	u8 currLine;
+	u8 currBgNum;
+	bool blend1;
+	u8* currDst;
+
+
+	static struct MosaicLookup {
+
+		struct TableEntry {
+			u8 begin, trunc;
+		} table[16][256];
+
+		MosaicLookup() {
+			for(int m=0;m<16;m++)
+				for(int i=0;i<256;i++) {
+					int mosaic = m+1;
+					TableEntry &te = table[m][i];
+					te.begin = (i%mosaic==0);
+					te.trunc = i/mosaic*mosaic;
+				}
+		}
+
+		TableEntry *width, *height;
+		int widthValue, heightValue;
+		
+	} mosaicLookup;
+	bool curr_mosaic_enabled;
+
+	u16 blend(u16 colA, u16 colB);
+
+	typedef void (*FinalOBJColFunct)(GPU *gpu, u32 passing, u8 *dst, u16 color, u8 alpha, u8 type, u16 x);
+	typedef void (*Final3DColFunct)(GPU *gpu, u32 passing, u8 *dst, u16 color, u8 alpha, u16 x);
+	typedef void (*SpriteRenderFunct) (GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
+
+	int setFinalColorBck_funcNum;
+	FinalOBJColFunct setFinalColorSpr;
+	Final3DColFunct setFinalColor3D;
+	SpriteRenderFunct spriteRender;
+
+	void setFinalColorBG(u16 color, u8 x);
+	FORCEINLINE void setFinalBGColorSpecialNone(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialBlend(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialIncrease(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialDecrease(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialNoneWnd(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialBlendWnd(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialIncreaseWnd(u16 color, u8 x, bool blend1);
+	FORCEINLINE void setFinalBGColorSpecialDecreaseWnd(u16 color, u8 x, bool blend1);
+
+	void __setFinalColorBck(u16 color, u8 x, bool opaque);
+	void setAffineStart(int layer, int xy, u32 val);
+
+	struct AffineInfo {
+		AffineInfo() : x(0), y(0) {}
+		u32 x, y;
+	} affineInfo[2];
+
+	void renderline_checkWindows(u16 x, bool &draw, bool &effect) const;
+
+	// check whether (x,y) is within the rectangle (including wraparounds) 
+	template<int WIN_NUM>
+	bool withinRect(u8 x) const;
+
+	void setBLDALPHA(u16 val)
+	{
+		BLDALPHA_EVA = (val&0x1f) > 16 ? 16 : (val&0x1f); 
+		BLDALPHA_EVB = (val>>8&0x1f) > 16 ? 16 : (val>>8&0x1f);
+		updateBLDALPHA();
+	}
+
+	void setBLDALPHA_EVA(u8 val)
+	{
+		BLDALPHA_EVA = (val&0x1f) > 16 ? 16 : (val&0x1f);
+		updateBLDALPHA();
+	}
+	
+	void setBLDALPHA_EVB(u8 val)
+	{
+		BLDALPHA_EVB = (val&0x1f) > 16 ? 16 : (val&0x1f);
+		updateBLDALPHA();
+	}
+
+	typedef u8 TBlendTable[32][32];
+	TBlendTable *blendTable;
+
+	void updateBLDALPHA()
+	{
+		blendTable = (TBlendTable*)&gpuBlendTable555[BLDALPHA_EVA][BLDALPHA_EVB][0][0];
+	}
+	
 };
 /*
 // normally should have same addresses
@@ -671,23 +794,26 @@ static void REG_DISPx_pack_test(GPU * gpu)
 }
 */
 
-extern u8 GPU_screen[4*256*192];
+CACHE_ALIGN extern u8 GPU_screen[4*256*192];
 
 
 GPU * GPU_Init(u8 l);
 void GPU_Reset(GPU *g, u8 l);
 void GPU_DeInit(GPU *);
 
-void textBG(const GPU * gpu, u8 num, u8 * DST);		//Draw text based background
-void rotBG(GPU * gpu, u8 num, u8 * DST);
-void extRotBG(GPU * gpu, u8 num, u8 * DST);
-void sprite1D(GPU * gpu, u16 l, u8 * dst, u8 * prioTab);
-void sprite2D(GPU * gpu, u16 l, u8 * dst, u8 * prioTab);
+//these are functions used by debug tools which want to render layers etc outside the context of the emulation
+namespace GPU_EXT
+{
+	void textBG(GPU * gpu, u8 num, u8 * DST);		//Draw text based background
+	void rotBG(GPU * gpu, u8 num, u8 * DST);
+	void extRotBG(GPU * gpu, u8 num, u8 * DST);
+};
+void sprite1D(GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
+void sprite2D(GPU * gpu, u16 l, u8 * dst, u8 * dst_alpha, u8 * typeTab, u8 * prioTab);
 
-extern short sizeTab[4][4][2];
-extern size sprSizeTab[4][4];
-extern s8 mode2type[8][4];
-extern void (*modeRender[8][4])(GPU * gpu, u8 num, u16 l, u8 * DST);
+extern const short sizeTab[4][4][2];
+extern const size sprSizeTab[4][4];
+extern const s8 mode2type[8][4];
 
 typedef struct {
 	GPU * gpu;
@@ -727,7 +853,6 @@ void GPU_setVideoProp(GPU *, u32 p);
 void GPU_setBGProp(GPU *, u16 num, u16 p);
 
 void GPU_setBLDCNT(GPU *gpu, u16 v) ;
-void GPU_setBLDALPHA(GPU *gpu, u16 v) ;
 void GPU_setBLDY(GPU *gpu, u16 v) ;
 void GPU_setMOSAIC(GPU *gpu, u16 v) ;
 
@@ -737,47 +862,54 @@ void GPU_addBack(GPU *, u8 num);
 
 int GPU_ChangeGraphicsCore(int coreid);
 
-void GPU_set_DISPCAPCNT(GPU * gpu, u32 val) ;
+void GPU_set_DISPCAPCNT(u32 val) ;
 void GPU_ligne(NDS_Screen * screen, u16 l) ;
 void GPU_setMasterBrightness (GPU *gpu, u16 val);
 
-void GPU_setWIN0_H	(GPU *gpu, u16 val);
-void GPU_setWIN0_H0 (GPU *gpu, u8 val);
-void GPU_setWIN0_H1 (GPU *gpu, u8 val);
+#define GPU_setWIN0_H(gpu, val) {gpu->WIN0H0 = val >> 8; gpu->WIN0H1 = val&0xFF; gpu->need_update_winh[0] = true; }
+#define GPU_setWIN0_H0(gpu, val) { gpu->WIN0H0 = val;  gpu->need_update_winh[0] = true; }
+#define GPU_setWIN0_H1(gpu, val) { gpu->WIN0H1 = val;  gpu->need_update_winh[0] = true; }
 
-void GPU_setWIN0_V	(GPU *gpu, u16 val);
-void GPU_setWIN0_V0 (GPU *gpu, u8 val);
-void GPU_setWIN0_V1 (GPU *gpu, u8 val);
+#define GPU_setWIN0_V(gpu, val) {gpu->WIN0V0 = val >> 8; gpu->WIN0V1 = val&0xFF;}
+#define GPU_setWIN0_V0(gpu, val) gpu->WIN0V0 = val
+#define GPU_setWIN0_V1(gpu, val) gpu->WIN0V1 = val
 
-void GPU_setWIN1_H	(GPU *gpu, u16 val);
-void GPU_setWIN1_H0 (GPU *gpu, u8 val);
-void GPU_setWIN1_H1 (GPU *gpu, u8 val);
+#define GPU_setWIN1_H(gpu, val) {gpu->WIN1H0 = val >> 8; gpu->WIN1H1 = val&0xFF;  gpu->need_update_winh[1] = true; }
+#define GPU_setWIN1_H0(gpu, val) { gpu->WIN1H0 = val;  gpu->need_update_winh[1] = true; }
+#define GPU_setWIN1_H1(gpu, val) { gpu->WIN1H1 = val;  gpu->need_update_winh[1] = true; }
 
-void GPU_setWIN1_V	(GPU *gpu, u16 val);
-void GPU_setWIN1_V0 (GPU *gpu, u8 val);
-void GPU_setWIN1_V1 (GPU *gpu, u8 val);
+#define GPU_setWIN1_V(gpu, val) {gpu->WIN1V0 = val >> 8; gpu->WIN1V1 = val&0xFF;}
+#define GPU_setWIN1_V0(gpu, val) gpu->WIN1V0 = val
+#define GPU_setWIN1_V1(gpu, val) gpu->WIN1V1 = val
 
-void GPU_setWININ  (GPU *gpu, u16 val);
-void GPU_setWININ0 (GPU *gpu, u8 val);
-void GPU_setWININ1 (GPU *gpu, u8 val);
+#define GPU_setWININ(gpu, val) {gpu->WININ0=val&0x1F;\
+								gpu->WININ0_SPECIAL=(val>>5)&1;\
+								gpu->WININ1=(val>>8)&0x1F;\
+								gpu->WININ1_SPECIAL=(val>>13)&1;\
+								}
+#define GPU_setWININ0(gpu, val) {gpu->WININ0 = val&0x1F; gpu->WININ0_SPECIAL = (val>>5)&1;}
+#define GPU_setWININ1(gpu, val) {gpu->WININ1 = val&0x1F; gpu->WININ1_SPECIAL = (val>>5)&1;}
 
-void GPU_setWINOUT16(GPU *gpu, u16 val);
-void GPU_setWINOUT	(GPU *gpu, u8 val);
-void GPU_setWINOBJ	(GPU *gpu, u8 val);
+#define GPU_setWINOUT16(gpu, val) { gpu->WINOUT=val&0x1F;\
+									gpu->WINOUT_SPECIAL=(val>>5)&1;\
+									gpu->WINOBJ=(val>>8)&0x1F;\
+									gpu->WINOBJ_SPECIAL=(val>>13)&1;\
+									}
+#define GPU_setWINOUT(gpu, val) {gpu->WINOUT = val&0x1F; gpu->WINOUT_SPECIAL = (val>>5)&1;}
+#define GPU_setWINOBJ(gpu, val) {gpu->WINOBJ = val&0x1F; gpu->WINOBJ_SPECIAL = (val>>5)&1;}
 
-void GPU_setBLDCNT_LOW	(GPU *gpu, u8 val);
-void GPU_setBLDCNT_HIGH (GPU *gpu, u8 val);
-void GPU_setBLDCNT		(GPU *gpu, u16 val);
+// Blending
+void SetupFinalPixelBlitter (GPU *gpu);
+#define GPU_setBLDCNT_LOW(gpu, val) {gpu->BLDCNT = (gpu->BLDCNT&0xFF00) | val; SetupFinalPixelBlitter (gpu);}
+#define GPU_setBLDCNT_HIGH(gpu, val) {gpu->BLDCNT = (gpu->BLDCNT&0xFF) | (val<<8); SetupFinalPixelBlitter (gpu);}
+#define GPU_setBLDCNT(gpu, val) {gpu->BLDCNT = val; SetupFinalPixelBlitter (gpu);}
 
-void GPU_setBLDALPHA	(GPU *gpu, u16 val);
-void GPU_setBLDALPHA_EVA(GPU *gpu, u8 val);
-void GPU_setBLDALPHA_EVB(GPU *gpu, u8 val);
 
-void GPU_setBLDY_EVY	(GPU *gpu, u8 val);
 
-#ifdef __cplusplus
-}
-#endif
+#define GPU_setBLDY_EVY(gpu, val) {gpu->BLDY_EVY = (val&0x1f) > 16 ? 16 : (val&0x1f);}
+
+#define GPU_setBGxHOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxHOFS = (val & 0x1F)
+#define GPU_setBGxVOFS(bg, gpu, val) gpu->dispx_st->dispx_BGxOFS[bg].BGxVOFS = (val & 0x1F)
 
 #endif
 
