@@ -31,13 +31,10 @@
 #include "mem.h"
 #include "wifi.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-
 extern volatile BOOL execute;
 extern BOOL click;
+extern char pathToROM[MAX_PATH];
+extern char pathFilenameToROMwithoutExt[MAX_PATH];
 
 /*
  * The firmware language values
@@ -55,7 +52,7 @@ extern BOOL click;
 //#define LOG_ARM9
 //#define LOG_ARM7
 
-typedef struct
+struct NDS_header
 {
        char     gameTile[12];
        char     gameCode[4];
@@ -104,25 +101,39 @@ typedef struct
        u16     logoCRC16;
        u16     headerCRC16;
        u8      reserved[160];
-} NDS_header;
+};
 
 extern void debug();
+void emu_halt();
 
 typedef struct
 {
-       s32 ARM9Cycle;
-       s32 ARM7Cycle;
-       s32 cycles;
-       s32 timerCycle[2][4];
-       BOOL timerOver[2][4];
-       s32 nextHBlank;
-       u32 VCount;
-       u32 old;
-       s32 diff;
-       BOOL lignerendu;
-       
-       u16 touchX;
-       u16 touchY;
+	s32 ARM9Cycle;
+	s32 ARM7Cycle;
+	s32 cycles;
+	s32 timerCycle[2][4];
+	BOOL timerOver[2][4];
+	s32 nextHBlank;
+	u32 VCount;
+	u32 old;
+	s32 diff;
+	BOOL lignerendu;
+
+	u16 touchX;
+	u16 touchY;
+	BOOL isTouch;
+	u16 pad;
+
+	//this is not essential NDS runtime state.
+	//it was perhaps a mistake to put it here.
+	//it is far less important than the above.
+	//maybe I should move it.
+	s32 idleCycles;
+	s32 runCycleCollector[16];
+	s32 idleFrameCounter;
+
+	bool isInVblank() const { return VCount >= 192; } 
+	bool isIn3dVblank() const { return VCount >= 192 && VCount<215; } 
 } NDSSystem;
 
 /** /brief A touchscreen calibration point.
@@ -139,7 +150,8 @@ struct NDS_fw_touchscreen_cal {
  */
 enum nds_fw_ds_type {
   NDS_FW_DS_TYPE_FAT,
-  NDS_FW_DS_TYPE_LITE
+  NDS_FW_DS_TYPE_LITE,
+  NDS_FW_DS_TYPE_iQue
 };
 
 #define MAX_FW_NICKNAME_LENGTH 10
@@ -183,10 +195,16 @@ BOOL NDS_SetROM(u8 * rom, u32 mask);
 NDS_header * NDS_getROMHeader(void);
  
 void NDS_setTouchPos(u16 x, u16 y);
-void NDS_releasTouch(void);
+void NDS_releaseTouch(void);
+void NDS_setPad(bool R,bool L,bool D,bool U,bool T,bool S,bool B,bool A,bool Y,bool X,bool W,bool E,bool G, bool F);
+void NDS_setPadFromMovie(u16 pad);
 
+#ifdef EXPERIMENTAL_GBASLOT
+int NDS_LoadROM(const char *filename, int bmtype, u32 bmsize);
+#else
 int NDS_LoadROM(const char *filename, int bmtype, u32 bmsize,
                  const char *cflash_disk_image_file);
+#endif
 void NDS_FreeROM(void);
 void NDS_Reset(void);
 int NDS_ImportSave(const char *filename);
@@ -194,14 +212,20 @@ int NDS_ImportSave(const char *filename);
 int NDS_WriteBMP(const char *filename);
 int NDS_LoadFirmware(const char *filename);
 int NDS_CreateDummyFirmware( struct NDS_fw_config_data *user_settings);
-u32
-NDS_exec(s32 nb, BOOL force);
+
+void NDS_SkipFrame(bool skip);
+
+template<bool FORCE>
+u32 NDS_exec(s32 nb);
+
+inline u32 NDS_exec(s32 nb) { return NDS_exec<false>(nb); }
 
        static INLINE void NDS_ARM9HBlankInt(void)
        {
             if(T1ReadWord(ARM9Mem.ARM9_REG, 4) & 0x10)
             {
-                 MMU.reg_IF[0] |= 2;// & (MMU.reg_IME[0] << 1);// (MMU.reg_IE[0] & (1<<1));
+                 //MMU.reg_IF[0] |= 2;// & (MMU.reg_IME[0] << 1);// (MMU.reg_IE[0] & (1<<1));
+				setIF(0, 2);
                  NDS_ARM9.wIRQ = TRUE;
             }
        }
@@ -210,7 +234,8 @@ NDS_exec(s32 nb, BOOL force);
        {
             if(T1ReadWord(MMU.ARM7_REG, 4) & 0x10)
             {
-                 MMU.reg_IF[1] |= 2;// & (MMU.reg_IME[1] << 1);// (MMU.reg_IE[1] & (1<<1));
+                // MMU.reg_IF[1] |= 2;// & (MMU.reg_IME[1] << 1);// (MMU.reg_IE[1] & (1<<1));
+				setIF(1, 2);
                  NDS_ARM7.wIRQ = TRUE;
             }
        }
@@ -219,9 +244,10 @@ NDS_exec(s32 nb, BOOL force);
        {
             if(T1ReadWord(ARM9Mem.ARM9_REG, 4) & 0x8)
             {
-                 MMU.reg_IF[0] |= 1;// & (MMU.reg_IME[0]);// (MMU.reg_IE[0] & 1);
+                // MMU.reg_IF[0] |= 1;// & (MMU.reg_IME[0]);// (MMU.reg_IE[0] & 1);
+				setIF(0, 1);
                  NDS_ARM9.wIRQ = TRUE;
-                      //execute = FALSE;
+                      //emu_halt();
                       /*logcount++;*/
             }
        }
@@ -229,9 +255,10 @@ NDS_exec(s32 nb, BOOL force);
        static INLINE void NDS_ARM7VBlankInt(void)
        {
             if(T1ReadWord(MMU.ARM7_REG, 4) & 0x8)
-                 MMU.reg_IF[1] |= 1;// & (MMU.reg_IME[1]);// (MMU.reg_IE[1] & 1);
+                // MMU.reg_IF[1] |= 1;// & (MMU.reg_IME[1]);// (MMU.reg_IE[1] & 1);
+				setIF(1, 1);
                  NDS_ARM7.wIRQ = TRUE;
-                 //execute = FALSE;
+                 //emu_halt();
        }
        
        static INLINE void NDS_swapScreen(void)
@@ -240,11 +267,8 @@ NDS_exec(s32 nb, BOOL force);
 	       MainScreen.offset = SubScreen.offset;
 	       SubScreen.offset = tmp;
        }
- 
- 
-#ifdef __cplusplus
-}
-#endif
+
+	int NDS_WriteBMP_32bppBuffer(int width, int height, const void* buf, const char *filename);
 
 #endif
 
