@@ -21,6 +21,7 @@
 #define SPU_H
 
 #include "types.h"
+#include <iosfwd>
 
 #define SNDCORE_DEFAULT         -1
 #define SNDCORE_DUMMY           0
@@ -42,8 +43,9 @@ typedef struct
 extern SoundInterface_struct SNDDummy;
 extern SoundInterface_struct SNDFile;
 
-typedef struct
+struct channel_struct
 {
+	int num;
    u8 vol;
    u8 datashift;
    u8 hold;
@@ -62,23 +64,34 @@ typedef struct
    double sampinc;
    // ADPCM specific
    int lastsampcnt;
-   s16 pcm16b;
+   s16 pcm16b, pcm16b_last;
    int index;
-} channel_struct;
+   u16 x;
+   s16 psgnoise_last;
+} ;
 
-typedef struct
+class SPU_struct
 {
+public:
+	SPU_struct(int buffersize);
    u32 bufpos;
    u32 buflength;
    s32 *sndbuf;
    s16 *outbuf;
    u32 bufsize;
-   channel_struct chan[16];
-} SPU_struct;
+   channel_struct channels[16];
 
-extern SPU_struct *SPU;
+   void reset();
+   ~SPU_struct();
+   void KeyOn(int channel);
+   void WriteByte(u32 addr, u8 val);
+   void WriteWord(u32 addr, u16 val);
+   void WriteLong(u32 addr, u32 val);
+};
 
 int SPU_ChangeSoundCore(int coreid, int buffersize);
+SoundInterface_struct *SPU_SoundCore();
+
 int SPU_Init(int coreid, int buffersize);
 void SPU_Pause(int pause);
 void SPU_SetVolume(int volume);
@@ -89,6 +102,13 @@ void SPU_WriteByte(u32 addr, u8 val);
 void SPU_WriteWord(u32 addr, u16 val);
 void SPU_WriteLong(u32 addr, u32 val);
 u32 SPU_ReadLong(u32 addr);
-void SPU_Emulate(void);
+void SPU_Emulate_core(void);
+void SPU_Emulate_user(void);
+
+extern SPU_struct *SPU_core;
+extern int spu_core_samples;
+
+void spu_savestate(std::ostream* os);
+bool spu_loadstate(std::istream* is);
 
 #endif
