@@ -26,32 +26,33 @@
 #include "bits.h"
 #include "MMU.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 #define ARMCPU_ARM7 1
 #define ARMCPU_ARM9 0
+#define ARMPROC (PROCNUM?NDS_ARM7:NDS_ARM9)
 
-#define CODE(i)     (((i)>>25)&0X7)
+#define CODE(i)     (((i)>>25)&0x7)
 #define OPCODE(i)   (((i)>>21)&0xF)
 #define SIGNEBIT(i) BIT_N(i,20)
- 
+
 #define INSTRUCTION_INDEX(i) ((((i)>>16)&0xFF0)|(((i)>>4)&0xF))
 
-#define ROR(i, j)   ((((u32)(i))>>(j)) | (((u32)(i))<<(32-(j))))
+inline u32 ROR(u32 i, u32 j)   { return ((((u32)(i))>>(j)) | (((u32)(i))<<(32-(j)))); }
 
-#define UNSIGNED_OVERFLOW(a,b,c) ((BIT31(a)&BIT31(b)) | \
-								  ((BIT31(a)|BIT31(b))&BIT31(~c)))
+template<typename T>
+inline T UNSIGNED_OVERFLOW(T a,T b,T c) { return ((BIT31(a)&BIT31(b)) | 
+												  ((BIT31(a)|BIT31(b))&BIT31(~c))); }
 
-#define UNSIGNED_UNDERFLOW(a,b,c) ((BIT31(~a)&BIT31(b)) | \
-								  ((BIT31(~a)|BIT31(b))&BIT31(c)))
+template<typename T>
+inline T UNSIGNED_UNDERFLOW(T a,T b,T c) { return ((BIT31(~a)&BIT31(b)) | 
+													((BIT31(~a)|BIT31(b))&BIT31(c))); }
 
-#define SIGNED_OVERFLOW(a,b,c) ((BIT31(a)&BIT31(b)&BIT31(~c))|\
-								(BIT31(~a)&BIT31(~(b))&BIT31(c)))
+template<typename T>
+inline T SIGNED_OVERFLOW(T a,T b,T c) { return ((BIT31(a)&BIT31(b)&BIT31(~c))|
+										  (BIT31(~a)&BIT31(~(b))&BIT31(c))); }
 
-#define SIGNED_UNDERFLOW(a,b,c) ((BIT31(a)&BIT31(~(b))&BIT31(~c))|\
-								(BIT31(~a)&BIT31(b)&BIT31(c)))
+template<typename T>
+inline T SIGNED_UNDERFLOW(T a,T b,T c) { return ((BIT31(a)&BIT31(~(b))&BIT31(~c))|
+										  (BIT31(~a)&BIT31(b)&BIT31(c))); }
 
 #define EQ	0x0
 #define NE	0x1
@@ -67,23 +68,11 @@ extern "C" {
 #define LT	0xB
 #define GT	0xC
 #define LE	0xD
-#define AL	0xE	
+#define AL	0xE
 
-#define TEST_COND(cond, CPSR)	(((cond)==AL) ||\
-					 (((cond)==EQ) && ( CPSR.bits.Z))||\
-					 (((cond)==NE) && (!CPSR.bits.Z))||\
-					 (((cond)==CS) && ( CPSR.bits.C))||\
-					 (((cond)==CC) && (!CPSR.bits.C))||\
-					 (((cond)==MI) && ( CPSR.bits.N))||\
-					 (((cond)==PL) && (!CPSR.bits.N))||\
-					 (((cond)==VS) && ( CPSR.bits.V))||\
-					 (((cond)==VC) && (!CPSR.bits.V))||\
-					 (((cond)==HI) && (CPSR.bits.C) && (!CPSR.bits.Z))||\
-					 (((cond)==LS) && ((CPSR.bits.Z) || (!CPSR.bits.C)))||\
-					 (((cond)==GE) && (CPSR.bits.N==CPSR.bits.V))||\
-					 (((cond)==LT) && (CPSR.bits.N!=CPSR.bits.V))||\
-					 (((cond)==GT) && (CPSR.bits.Z==0) && (CPSR.bits.N==CPSR.bits.V))||\
-					 (((cond)==LE) && ((CPSR.bits.Z) || (CPSR.bits.N!=CPSR.bits.V))))
+extern const unsigned char arm_cond_table[16*16];
+
+#define TEST_COND(cond, inst, CPSR)   ((arm_cond_table[((CPSR.val >> 24) & 0xf0)+(cond)] >> (inst)) & 1)
 
 
 enum Mode
@@ -98,13 +87,13 @@ enum Mode
 };
 
 #ifdef WORDS_BIGENDIAN
-typedef union 
+typedef union
 {
-	struct 
+	struct
 	{
 		u32 N : 1,
-		Z : 1, 
-		C : 1, 
+		Z : 1,
+		C : 1,
 		V : 1,
 		Q : 1,
 		RAZ : 19,
@@ -116,9 +105,9 @@ typedef union
         u32 val;
 } Status_Reg;
 #else
-typedef union 
+typedef union
 {
-	struct 
+	struct
 	{
                 u32 mode : 5,
 		T : 1,
@@ -127,8 +116,8 @@ typedef union
 		RAZ : 19,
 		Q : 1,
 		V : 1,
-		C : 1, 
-		Z : 1, 
+		C : 1,
+		Z : 1,
 		N : 1;
 	} bits;
         u32 val;
@@ -172,11 +161,11 @@ typedef struct armcpu_t
         u32 instruction; //4
         u32 instruct_adr; //8
         u32 next_instruction; //12
-       
+
         u32 R[16]; //16
 	Status_Reg CPSR;  //80
 	Status_Reg SPSR;
-       
+
         u32 R13_usr, R14_usr;
         u32 R13_svc, R14_svc;
         u32 R13_abt, R14_abt;
@@ -184,17 +173,20 @@ typedef struct armcpu_t
         u32 R13_irq, R14_irq;
         u32 R8_fiq, R9_fiq, R10_fiq, R11_fiq, R12_fiq, R13_fiq, R14_fiq;
 	Status_Reg SPSR_svc, SPSR_abt, SPSR_und, SPSR_irq, SPSR_fiq;
-	
+
 	armcp_t *coproc[16];
-	       
+
         u32 intVector;
         u8 LDTBit;  //1 : ARMv5 style 0 : non ARMv5
 	BOOL waitIRQ;
 	BOOL wIRQ;
 	BOOL wirq;
 
-        u32 (* *swi_tab)(struct armcpu_t * cpu);
+	u32 newIrqFlags;
 
+        u32 (* *swi_tab)();
+
+#ifdef GDB_STUB
   /** there is a pending irq for the cpu */
   int irq_flag;
 
@@ -204,6 +196,7 @@ typedef struct armcpu_t
   /** data for the post executed function */
   void *post_ex_fn_data;
 
+
   /** flag indicating if the processor is stalled */
   int stalled;
 
@@ -212,27 +205,41 @@ typedef struct armcpu_t
 
   /** the ctrl interface */
   struct armcpu_ctrl_iface ctrl_iface;
+#endif
 } armcpu_t;
 
-
+#ifdef GDB_STUB
 int armcpu_new( armcpu_t *armcpu, u32 id, struct armcpu_memory_iface *mem_if,
                 struct armcpu_ctrl_iface **ctrl_iface_ret);
+#else
+int armcpu_new( armcpu_t *armcpu, u32 id);
+#endif
 void armcpu_init(armcpu_t *armcpu, u32 adr);
 u32 armcpu_switchMode(armcpu_t *armcpu, u8 mode);
-//u32 armcpu_prefetch(armcpu_t *armcpu);
-u32 armcpu_exec(armcpu_t *armcpu);
-//BOOL armcpu_irqExeption(armcpu_t *armcpu);
-//BOOL armcpu_prefetchExeption(armcpu_t *armcpu);
-BOOL
-armcpu_flagIrq( armcpu_t *armcpu);
+
+
+template<int PROCNUM> u32 armcpu_exec();
+
+BOOL armcpu_irqException(armcpu_t *armcpu);
+BOOL armcpu_flagIrq( armcpu_t *armcpu);
 
 extern armcpu_t NDS_ARM7;
 extern armcpu_t NDS_ARM9;
 
+
+static INLINE void setIF(int PROCNUM, u32 flag)
+{
+	MMU.reg_IF[PROCNUM] |= flag;
+
+	if(ARMPROC.waitIRQ)
+		ARMPROC.newIrqFlags |= flag;
+}
+
 static INLINE void NDS_makeARM9Int(u32 num)
 {
         /* flag the interrupt request source */
-        MMU.reg_IF[0] |= (1<<num);
+       // MMU.reg_IF[0] |= (1<<num);
+		setIF(0, (1<<num));
 
         /* generate the interrupt if enabled */
 	if ((MMU.reg_IE[0] & (1 << num)) && MMU.reg_IME[0])
@@ -241,11 +248,12 @@ static INLINE void NDS_makeARM9Int(u32 num)
 		NDS_ARM9.waitIRQ = FALSE;
 	}
 }
-       
+
 static INLINE void NDS_makeARM7Int(u32 num)
 {
         /* flag the interrupt request source */
-	MMU.reg_IF[1] |= (1<<num);
+	//MMU.reg_IF[1] |= (1<<num);
+	setIF(1, (1<<num));
 
         /* generate the interrupt if enabled */
 	if ((MMU.reg_IE[1] & (1 << num)) && MMU.reg_IME[1])
@@ -268,9 +276,4 @@ static INLINE void NDS_makeInt(u8 proc_ID,u32 num)
 	}
 }
 
-
-#ifdef __cplusplus
-}
 #endif
-
-#endif 
