@@ -17,119 +17,164 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#import <Cocoa/Cocoa.h>
-#import <OpenGL/gl.h>
-
 //DeSmuME Cocoa includes
 #import "main_window.h"
 #import "screenshot.h"
+#import "screen_state.h"
 #import "video_output_view.h"
 #import "input.h"
-#import "rom_info.h"
 #import "preferences.h"
+#import "dialogs/rom_info.h"
+#import "dialogs/speed_limit_selection_window.h"
 
 //How much padding to put around the video output
 #define WINDOW_BORDER_PADDING 5
 
 #define MAX_FRAME_SKIP 10
 
-#define ROTATION_0   0
-#define ROTATION_90  1
-#define ROTATION_180 2
-#define ROTATION_270 3
-
 #define DS_SCREEN_HEIGHT_COMBINED (192*2) /*height of the two screens*/
 #define DS_SCREEN_X_RATIO (256.0 / (192.0 * 2.0))
 #define DS_SCREEN_Y_RATIO ((192.0 * 2.0) / 256.0)
 
 //
-NSMenuItem *close_rom_item;
-NSMenuItem *execute_item;
-NSMenuItem *pause_item;
-NSMenuItem *reset_item;
-NSMenuItem *save_state_as_item;
-NSMenuItem *load_state_from_item;
-NSMenuItem *saveSlot_item[MAX_SLOTS];
-NSMenuItem *loadSlot_item[MAX_SLOTS];
-//NSMenuItem *clear_all_saves_item;
-NSMenuItem *rom_info_item;
-NSMenuItem *frame_skip_auto_item;
-NSMenuItem *frame_skip_item[MAX_FRAME_SKIP];
+NSMenuItem *close_rom_item = nil;
+NSMenuItem *execute_item = nil;
+NSMenuItem *pause_item = nil;
+NSMenuItem *reset_item = nil;
+NSMenuItem *save_state_as_item = nil;
+NSMenuItem *load_state_from_item = nil;
+NSMenuItem *saveSlot_item[MAX_SLOTS] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil }; //make sure this corresponds to the amount in max slots
+NSMenuItem *loadSlot_item[MAX_SLOTS] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil };
+NSMenuItem *rom_info_item = nil;
+NSMenuItem *frame_skip_auto_item = nil;
+NSMenuItem *frame_skip_item[MAX_FRAME_SKIP] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil };
+NSMenuItem *speed_limit_25_item = nil;
+NSMenuItem *speed_limit_50_item = nil;
+NSMenuItem *speed_limit_75_item = nil;
+NSMenuItem *speed_limit_100_item = nil;
+NSMenuItem *speed_limit_200_item = nil;
+NSMenuItem *speed_limit_none_item = nil;
+NSMenuItem *speed_limit_custom_item = nil;
 
-NSMenuItem *volume_item[10];
-NSMenuItem *mute_item;
+NSMenuItem *volume_item[10] = { nil, nil, nil, nil, nil, nil, nil, nil, nil, nil };
+NSMenuItem *mute_item = nil;
 
 //screen menu items
-NSMenuItem *resize1x;
-NSMenuItem *resize2x;
-NSMenuItem *resize3x;
-NSMenuItem *resize4x;
-NSMenuItem *constrain_item;
-NSMenuItem *min_size_item;
-NSMenuItem *toggle_status_bar_item;
-NSMenuItem *rotation0_item;
-NSMenuItem *rotation90_item;
-NSMenuItem *rotation180_item;
-NSMenuItem *rotation270_item;
-NSMenuItem *topBG0_item;
-NSMenuItem *topBG1_item;
-NSMenuItem *topBG2_item;
-NSMenuItem *topBG3_item;
-NSMenuItem *subBG0_item;
-NSMenuItem *subBG1_item;
-NSMenuItem *subBG2_item;
-NSMenuItem *subBG3_item;
-NSMenuItem *screenshot_to_file_item;
+NSMenuItem *resize1x = nil;
+NSMenuItem *resize2x = nil;
+NSMenuItem *resize3x = nil;
+NSMenuItem *resize4x = nil;
+NSMenuItem *constrain_item = nil;
+NSMenuItem *min_size_item = nil;
+NSMenuItem *toggle_status_bar_item = nil;
+NSMenuItem *rotation0_item = nil;
+NSMenuItem *rotation90_item = nil;
+NSMenuItem *rotation180_item = nil;
+NSMenuItem *rotation270_item = nil;
+NSMenuItem *topBG0_item = nil;
+NSMenuItem *topBG1_item = nil;
+NSMenuItem *topBG2_item = nil;
+NSMenuItem *topBG3_item = nil;
+NSMenuItem *subBG0_item = nil;
+NSMenuItem *subBG1_item = nil;
+NSMenuItem *subBG2_item = nil;
+NSMenuItem *subBG3_item = nil;
+NSMenuItem *screenshot_to_file_item = nil;
 
 @implementation VideoOutputWindow
 
+//Private interface ------------------------------------------------------------
+
 - (void)setStatusText:(NSString*)value
 {
-	static NSString *retained_value = nil; //stores the string value incease the status_view is deleted and recreated
+	[status_bar_text release];
+	status_bar_text = value;
+	[status_bar_text retain];
 
-	if(status_view == nil)
+	if(status_view)[status_view setStringValue:status_bar_text];
+}
+
+- (NSString*)statusText
+{
+	if(status_bar_text)return status_bar_text;
+	return @"";
+}
+
+- (float)statusBarHeight
+{
+	if(status_view == nil)return WINDOW_BORDER_PADDING;
+	NSRect frame = [status_view frame];
+	return frame.origin.y + frame.size.height;
+}
+
+- (void)resetMinSize:(BOOL)resize_if_too_small
+{
+	//keep the min size item up to date, just in case
+	if([min_size_item target] == self)
+		[min_size_item setState:no_smaller_than_ds?NSOnState:NSOffState];
+
+	NSSize min_size;
+
+	if(!no_smaller_than_ds)
 	{
-		retained_value = value;
-		[retained_value retain];
+		min_size.width = 0;
+		min_size.height = 0;
+	} else if(video_output_view == nil)
+	{
+		min_size.width = DS_SCREEN_WIDTH;
+		min_size.height = 0;
+	} else if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
+	{
+		min_size.width = DS_SCREEN_WIDTH;
+		min_size.height = DS_SCREEN_HEIGHT_COMBINED;
 	} else
 	{
-		if(value == nil)
-		{
-			if(retained_value != nil)
-			{
-				[status_view setStringValue:retained_value];
-			} else
-				[status_view setStringValue:@""];
-		} else
-		{
-			[status_view setStringValue:value];
+		min_size.width = DS_SCREEN_HEIGHT_COMBINED;
+		min_size.height = DS_SCREEN_WIDTH;
+	}
 
-			[retained_value release];
-			retained_value = value;
-			[retained_value retain];
-		}
+	//
+	NSSize adjusted_min_size = min_size;
+	adjusted_min_size.width += WINDOW_BORDER_PADDING * 2;
+	adjusted_min_size.height += [self statusBarHeight];
+
+	//set min content size for the window
+	[window setContentMinSize:adjusted_min_size];
+
+	if(resize_if_too_small)
+	{
+		NSSize temp = [[window contentView] frame].size;
+		if(temp.width < adjusted_min_size.width && temp.height >= adjusted_min_size.height)
+			[self resizeScreen:NSMakeSize(min_size.width, temp.height)];
+		else if(temp.width >= adjusted_min_size.width && temp.height < adjusted_min_size.height)
+			[self resizeScreen:NSMakeSize(temp.width, min_size.height)];
+		else if(temp.width < adjusted_min_size.width && temp.height < adjusted_min_size.height)
+			[self resizeScreen:NSMakeSize(min_size.width, min_size.height)];
 	}
 }
 
+//Public Interface -----------------------------------------------------------
+
 - (id)init
 {
 	//Create the NDS
 	self = [super init];
-
-	if(self == nil)return nil;
-
-	//
-	NSRect rect;
+	if(self == nil)return nil; //superclass will display it's own error messages if needed
 
 	//Set vars
+	controller = nil;
+	video_output_view = nil;
+	status_view = nil;
+	status_bar_text = nil;
+	input = nil;
 	no_smaller_than_ds = true;
 	keep_proportions = true;
 
 	//Create the window rect (content rect - title bar not included in size)
+	NSRect rect;
 	rect.size.width = DS_SCREEN_WIDTH + WINDOW_BORDER_PADDING * 2;
-	rect.size.height = DS_SCREEN_HEIGHT_COMBINED + status_bar_height;
-	rect.origin.x = 200;
-	rect.origin.y = 200;
+	rect.size.height = DS_SCREEN_HEIGHT_COMBINED + [self statusBarHeight];
+	rect.origin.x = rect.origin.y = 200;
 
 	//allocate the window
 	if((window = [[NSWindow alloc] initWithContentRect:rect styleMask:
@@ -137,53 +182,55 @@ NSMenuItem *screenshot_to_file_item;
 	backing:NSBackingStoreBuffered defer:NO screen:nil])==nil)
 	{
 		messageDialog(NSLocalizedString(@"Error", nil), @"Couldn't create window");
+		[super release];
 		return nil;
 	}
+
 	[window setTitle:NSLocalizedString(@"DeSmuME Emulator", nil)];
+	[window setShowsResizeIndicator:NO];
 
 	//Create the image view where we will display video output
 	rect.origin.x = WINDOW_BORDER_PADDING;
-	rect.origin.y = status_bar_height;
-	rect.size.width = DS_SCREEN_WIDTH;
-	rect.size.height = DS_SCREEN_HEIGHT_COMBINED;
-
-	if((video_output_view = [[VideoOutputView alloc] initWithFrame:rect withDS:self]) == nil)
-		messageDialog(NSLocalizedString(@"Error", nil), @"Couldn't create video output view");
+	rect.origin.y = [self statusBarHeight];
+	rect.size.width -= WINDOW_BORDER_PADDING*2;
+	rect.size.height -= [self statusBarHeight];
+	video_output_view = [[VideoOutputView alloc] initWithFrame:rect]; //no nil check - will do it's own error messages
+	[video_output_view setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable]; //view will automatically resize with the window
 
 	[[window contentView] addSubview:video_output_view];
 
-	//Add the video update callback
-	[self setVideoUpdateCallback:@selector(updateScreen:) withObject:video_output_view];
+	//create the status bar
+	[self setStatusText:NSLocalizedString(@"No ROM loaded", nil)];
+	[self toggleStatusBar];
 
-	//Add the error callback
+	//set min size
+	[self resetMinSize:YES];
+
+	//Add callbacks
+	[self setVideoUpdateCallback:@selector(setScreenState:) withObject:video_output_view];
 	[self setErrorCallback:@selector(emulationError) withObject:self];
 
 	//Show the window
 	[window setDelegate:self]; //we do this after making the ouput/statusbar incase some delegate method gets called suddenly
 	[controller = [[NSWindowController alloc] initWithWindow:window] showWindow:nil];
 
-	//Create the input manager
+	//Create the input manager and insert it into the cocoa responder chain
 	input = [[InputHandler alloc] initWithWindow:(id)self];
 	NSResponder *temp = [window nextResponder];
 	[window setNextResponder:input];
 	[input setNextResponder:temp];
 
-	//Start the status bar by default
-	//also sets the min window size
-	status_view = nil;
-	[self setStatusText:NSLocalizedString(@"No ROM loaded", nil)];
-	[self toggleStatusBar];
-
 	return self;
 }
 
 - (void)dealloc
 {
+	[controller release];
+	[window release];
 	[video_output_view release];
 	[status_view release];
-
-	[self retain]; //see the comment in init after we initialize the window controller
-	[controller release];
+	[status_bar_text release];
+	[input release];
 
 	[super dealloc];
 }
@@ -196,7 +243,7 @@ NSMenuItem *screenshot_to_file_item;
 	//Set status var and screen depending on whether the rom could be loaded
 	if(result == NO)
 	{
-		[video_output_view clearScreenBlack]; //black if the rom doesn't load
+		[video_output_view setScreenState:[ScreenState blackScreenState]]; //black if the rom doesn't load
 		[self setStatusText:NSLocalizedString(@"No ROM Loaded", nil)];
 		messageDialog(NSLocalizedString(@"Error", nil), @"Couldn't load ROM");
 	} else
@@ -206,7 +253,7 @@ NSMenuItem *screenshot_to_file_item;
 			[self execute];
 		else
 		{
-			[video_output_view clearScreenWhite]; //white if the rom loaded but is not playing
+			[video_output_view setScreenState:[ScreenState whiteScreenState]]; //white if the rom loaded but is not playing
 			[self setStatusText:NSLocalizedString(@"ROM Loaded", nil)];
 		}
 
@@ -222,15 +269,17 @@ NSMenuItem *screenshot_to_file_item;
 	{
 		int i;
 		for(i = 0; i < MAX_SLOTS; i++)
-			if([self saveStateExists:i] == YES)
-				[saveSlot_item[i] setState:NSOnState];
-			else
-				[saveSlot_item[i] setState:NSOffState];
+			if([saveSlot_item[i] target] == self)
+				if([self saveStateExists:i] == YES)
+					[saveSlot_item[i] setState:NSOnState];
+				else
+					[saveSlot_item[i] setState:NSOffState];
+
 	} else
 	{
 		int i;
 		for(i = 0; i < MAX_SLOTS; i++)
-			[saveSlot_item[i] setState:NSOffState];
+			if([saveSlot_item[i] target] == self)[saveSlot_item[i] setState:NSOffState];
 	}
 
 	return result;
@@ -269,7 +318,7 @@ NSMenuItem *screenshot_to_file_item;
 	if([self paused])
 	{
 		[self setStatusText:NSLocalizedString(@"Emulation Reset", nil)];
-		[video_output_view clearScreenWhite];
+		[video_output_view setScreenState:[ScreenState whiteScreenState]];
 	}
 }
 
@@ -318,12 +367,75 @@ NSMenuItem *screenshot_to_file_item;
 		}
 }
 
+- (void)setSpeedLimit:(int)speedLimit
+{
+	[super setSpeedLimit:speedLimit];
+
+	//Set the correct menu states
+	speedLimit = [super speedLimit];
+	int standard_size = 0;
+
+	if([speed_limit_25_item target] == self)
+		if(speedLimit == 25){ [speed_limit_25_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_25_item setState:NSOffState];
+
+	if([speed_limit_50_item target] == self)
+		if(speedLimit == 50){ [speed_limit_50_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_50_item setState:NSOffState];
+
+	if([speed_limit_75_item target] == self)
+		if(speedLimit == 75){ [speed_limit_75_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_75_item setState:NSOffState];
+
+	if([speed_limit_100_item target] == self)
+		if(speedLimit == 100){ [speed_limit_100_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_100_item setState:NSOffState];
+
+	if([speed_limit_200_item target] == self)
+		if(speedLimit == 200){ [speed_limit_200_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_200_item setState:NSOffState];
+
+	if([speed_limit_none_item target] == self)
+		if(speedLimit == 0){ [speed_limit_none_item setState:NSOnState]; standard_size=1; }
+		else [speed_limit_none_item setState:NSOffState];
+
+	if([speed_limit_custom_item target] == self)
+		if(!standard_size)[speed_limit_custom_item setState:NSOnState];
+		else [speed_limit_custom_item setState:NSOffState];
+
+}
+
+- (void)setSpeedLimitFromMenuItem:(id)sender
+{
+	if(sender == speed_limit_25_item  )[self setSpeedLimit:  25];
+	if(sender == speed_limit_50_item  )[self setSpeedLimit:  50];
+	if(sender == speed_limit_75_item  )[self setSpeedLimit:  75];
+	if(sender == speed_limit_100_item )[self setSpeedLimit: 100];
+	if(sender == speed_limit_200_item )[self setSpeedLimit: 200];
+	if(sender == speed_limit_none_item)[self setSpeedLimit:   0];
+
+	if(sender == speed_limit_custom_item)
+	{
+		//create
+		SpeedLimitSelectionWindow *s_window = [[SpeedLimitSelectionWindow alloc] initWithDS:self];
+
+		//show & run
+		NSWindowController *window_controller = [[NSWindowController alloc] initWithWindow:s_window];
+		[window_controller showWindow:nil];
+		[s_window runModal];
+
+		//release
+		[s_window release];
+		[window_controller release];
+	}
+}
+
 - (void)closeROM
 {
 	[super closeROM];
 
 	//reset window
-	[video_output_view clearScreenBlack];
+	[video_output_view setScreenState:[ScreenState blackScreenState]];
 	[self setStatusText:NSLocalizedString(@"No ROM Loaded", nil)];
 
 	//reset menu items
@@ -380,12 +492,24 @@ NSMenuItem *screenshot_to_file_item;
 
 - (BOOL)saveStateAs
 {
+    //pause emulation so it doesnt save the state after
+	BOOL was_executing = [self executing];
+	[self pause];
+
+	//file select
 	NSSavePanel *panel = [NSSavePanel savePanel];
 	[panel setTitle:NSLocalizedString(@"Save State...", nil)];
 	[panel setRequiredFileType:@"DST"];
 
+	//save it
 	if([panel runModal] == NSFileHandlingPanelOKButton)
+	{
+		if(was_executing == YES)[self execute];
 		return [self saveState:[panel filename]];
+	}
+
+	//unpause emulation if needed
+	if(was_executing == YES)[self execute];
 
 	return NO;
 }
@@ -399,7 +523,7 @@ NSMenuItem *screenshot_to_file_item;
 	[panel setAllowsMultipleSelection:NO];
 
 	if([panel runModalForTypes:[NSArray arrayWithObject:@"DST"]] == NSFileHandlingPanelOKButton)
-		return [self saveState:[panel filename]];
+		return [self loadState:[panel filename]];
 
 	return NO;
 }
@@ -428,57 +552,18 @@ NSMenuItem *screenshot_to_file_item;
 	return [self loadStateFromSlot:i];
 }
 
-- (void)resetMinSize
-{
-	//keep the min size item up to date, just in case
-	if([min_size_item target] == self)
-		[min_size_item setState:no_smaller_than_ds?NSOnState:NSOffState];
-
-	if(!no_smaller_than_ds)
-	{
-		[window setMinSize:NSMakeSize(100, 100)];
-		[window setContentMinSize:NSMakeSize(100, 100)];
-		return;
-	}
-
-	NSSize min_size;
-
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
-	{
-		min_size.width = DS_SCREEN_WIDTH + WINDOW_BORDER_PADDING * 2;
-		min_size.height = DS_SCREEN_HEIGHT_COMBINED + status_bar_height;
-		[window setContentMinSize:min_size];
-
-		//resize if too small
-		NSSize temp = [video_output_view frame].size;
-		if(temp.width < DS_SCREEN_WIDTH || temp.height < DS_SCREEN_HEIGHT_COMBINED)
-			[self resizeScreen:NSMakeSize(DS_SCREEN_WIDTH, DS_SCREEN_HEIGHT_COMBINED)]; //this could be per axis?
-
-	} else
-	{
-		min_size.width = DS_SCREEN_HEIGHT_COMBINED + WINDOW_BORDER_PADDING * 2;
-		min_size.height = DS_SCREEN_WIDTH + status_bar_height;
-		[window setContentMinSize:min_size];
-
-		//resize if too small
-		NSSize temp = [video_output_view frame].size;
-		if(temp.width < DS_SCREEN_HEIGHT_COMBINED || temp.height < DS_SCREEN_WIDTH)
-			[self resizeScreen:NSMakeSize(DS_SCREEN_HEIGHT_COMBINED, DS_SCREEN_WIDTH)];
-	}
-}
-
 - (void)toggleMinSize
 {
 	no_smaller_than_ds = !no_smaller_than_ds;
 
-	[self resetMinSize];
+	[self resetMinSize:YES];
 }
 
 - (void)resizeScreen:(NSSize)size
 {
 	//add the window borders
 	size.width += WINDOW_BORDER_PADDING * 2;
-	size.height += status_bar_height;
+	size.height += [self statusBarHeight];
 
 	//convert size to window frame rather than content frame
 	size = [window frameRectForContentRect:NSMakeRect(0, 0, size.width, size.height)].size;
@@ -501,8 +586,9 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)resizeScreen1x
 {
+	if(video_output_view == nil)return;
 
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+	if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 		[self resizeScreen:NSMakeSize(DS_SCREEN_WIDTH, DS_SCREEN_HEIGHT_COMBINED)];
 	else
 		[self resizeScreen:NSMakeSize(DS_SCREEN_HEIGHT_COMBINED, DS_SCREEN_WIDTH)];
@@ -510,8 +596,9 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)resizeScreen2x
 {
+	if(video_output_view == nil)return;
 
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+	if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 		[self resizeScreen:NSMakeSize(DS_SCREEN_WIDTH * 2, DS_SCREEN_HEIGHT_COMBINED * 2)];
 	else
 		[self resizeScreen:NSMakeSize(DS_SCREEN_HEIGHT_COMBINED * 2, DS_SCREEN_WIDTH * 2)];
@@ -519,7 +606,9 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)resizeScreen3x
 {
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+	if(video_output_view == nil)return;
+
+	if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 		[self resizeScreen:NSMakeSize(DS_SCREEN_WIDTH * 3, DS_SCREEN_HEIGHT_COMBINED * 3)];
 	else
 		[self resizeScreen:NSMakeSize(DS_SCREEN_HEIGHT_COMBINED * 3, DS_SCREEN_WIDTH * 3)];
@@ -527,7 +616,9 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)resizeScreen4x
 {
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+	if(video_output_view == nil)return;
+
+	if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 		[self resizeScreen:NSMakeSize(DS_SCREEN_WIDTH * 4, DS_SCREEN_HEIGHT_COMBINED * 4)];
 	else
 		[self resizeScreen:NSMakeSize(DS_SCREEN_HEIGHT_COMBINED * 4, DS_SCREEN_WIDTH * 4)];
@@ -535,24 +626,26 @@ NSMenuItem *screenshot_to_file_item;
 
 - (NSPoint)windowPointToDSCoords:(NSPoint)location
 {
+	if(video_output_view == nil)return NSMakePoint(-1, -1);
+
 	NSSize temp = [[window contentView] frame].size;
 	CGFloat x_size = temp.width - WINDOW_BORDER_PADDING*2;
-	CGFloat y_size = temp.height - status_bar_height;
+	CGFloat y_size = temp.height - [self statusBarHeight];
 
 	//If the click was to the left or under the opengl view, exit
-	if((location.x < WINDOW_BORDER_PADDING) || (location.y < status_bar_height))
+	if((location.x < WINDOW_BORDER_PADDING) || (location.y < [self statusBarHeight]))
 		return NSMakePoint(-1, -1);
-		
+
 	location.x -= WINDOW_BORDER_PADDING;
-	location.y -= status_bar_height;
+	location.y -= [self statusBarHeight];
 
 	//If the click was top or right of the view...
 	if(location.x >= x_size)return NSMakePoint(-1, -1);
 	if(location.y >= y_size)return NSMakePoint(-1, -1);
 
-	if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+	if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 	{
-		if([video_output_view rotation] == ROTATION_180)
+		if([video_output_view boundsRotation] == -180)
 		{
 			if(location.y < y_size / 2)return NSMakePoint(-1, -1);
 			location.x = x_size - location.x;
@@ -570,7 +663,7 @@ NSMenuItem *screenshot_to_file_item;
 	} else
 	{
 
-		if([video_output_view rotation] == ROTATION_270)
+		if([video_output_view boundsRotation] == -270)
 		{
 			if(location.x < x_size / 2)return NSMakePoint(-1, -1);
 			location.x = x_size - location.x;
@@ -605,7 +698,7 @@ NSMenuItem *screenshot_to_file_item;
 	{
         NSSize temp = [[window contentView] frame].size;
         temp.width -= WINDOW_BORDER_PADDING*2;
-        temp.height -= status_bar_height;
+        temp.height -= [self statusBarHeight];
 		[self resizeScreen:temp];
 	}
 
@@ -620,10 +713,10 @@ NSMenuItem *screenshot_to_file_item;
 	if(status_view == nil)
 	{
 		//Create the status view
-		rect.origin.x = WINDOW_BORDER_PADDING;
-		rect.origin.y = 0;
-		rect.size.width = [video_output_view frame].size.width;
-		rect.size.height = 1;
+
+//fixme subtract resize handle size to avoid overlap?
+		rect = NSMakeRect(WINDOW_BORDER_PADDING, 0, [window frame].size.width - WINDOW_BORDER_PADDING*2, 16);
+
 		status_view = [[NSTextField alloc] initWithFrame:rect];
 
 		if(status_view != nil)
@@ -632,41 +725,25 @@ NSMenuItem *screenshot_to_file_item;
 			[status_view setDrawsBackground:NO];
 			[status_view setBordered:NO];
 			[status_view setBezeled:NO];
-			[status_view setDrawsBackground:NO];
 			[status_view setSelectable:NO];
 			[status_view setEditable:NO];
-			[self setStatusText:nil];
-			[status_view sizeToFit];
+			[status_view setStringValue:[self statusText]];
+			[status_view setAutoresizingMask:NSViewWidthSizable];
 			rect = [status_view frame];
 
-			//fixme we need to determine the height of the resize handle
-			//this is a hack since tiger and earlier have a larger resize handle
-			if([NSWindow instancesRespondToSelector:@selector(setPreferredBackingLocation:)] == YES)
-				status_bar_height = rect.size.height + rect.origin.y + 2;
-			else
-				status_bar_height = rect.size.height + rect.origin.y + 6;
-
-			//move the video output view up
-			rect = [video_output_view frame];
-			rect.origin.y = status_bar_height;
-			[video_output_view setFrame:rect];
-
 			//add status view to the window
 			[[window contentView] addSubview:status_view];
 
-			//set the check
+			//set the menu checkmark
 			if([toggle_status_bar_item target] == self)
 			[toggle_status_bar_item setState:NSOnState];
-			
+
 			[window setShowsResizeIndicator:YES];
 
 		} else
 		{
 			messageDialog(NSLocalizedString(@"Error", nil), @"Couldn't create status bar");
-
-			if([toggle_status_bar_item target] == self)
-				[toggle_status_bar_item setState:NSOffState]; //should already be off, but just incase
-
+			return;
 		}
 
 	} else
@@ -675,185 +752,85 @@ NSMenuItem *screenshot_to_file_item;
 		[status_view removeFromSuperview];
 		[status_view release];
 		status_view = nil;
-		status_bar_height = WINDOW_BORDER_PADDING;
 
-		//move the video output view down
-		rect = [video_output_view frame];
-		rect.origin.y = WINDOW_BORDER_PADDING;
-		[video_output_view setFrame:rect];
-
-		//unset the check
+		//unset the menu checkmark
 		if([toggle_status_bar_item target] == self)
 			[toggle_status_bar_item setState:NSOffState];
-			
+
 		//hide the resize control (so it doesn't overlap our screen)
 		[window setShowsResizeIndicator:NO];
 	}
 
+	//move the video output view up or down to where the status bar is (or isn't)
+	rect = [video_output_view frame];
+	rect.origin.y = [self statusBarHeight];
+	[video_output_view setFrame:rect];
+
 	//resize the window
-	CGFloat new_height = [window frameRectForContentRect:NSMakeRect(0, 0, [[window contentView] frame].size.width, [video_output_view frame].size.height + status_bar_height)].size.height;
+	NSUInteger autoresizing_mask = [video_output_view autoresizingMask];
+	[video_output_view setAutoresizingMask:NSViewNotSizable]; //dont resize the video output when we resize the window\
+
+	CGFloat new_height = [window frameRectForContentRect:NSMakeRect(0, 0, [[window contentView] frame].size.width, [self statusBarHeight] + (video_output_view==nil?0:[video_output_view frame].size.height))].size.height;
+
 	rect = [window frame];
 	rect.origin.y += rect.size.height - new_height;
 	rect.size.height = new_height;
 	[window setFrame:rect display:YES];
 
-	//set window min size
-	[self resetMinSize];
+	[video_output_view setAutoresizingMask:autoresizing_mask];
+
+	//set new window min size
+	[self resetMinSize:YES];
 }
 
-- (void)setRotation0
+- (void)setRotation:(float)rotation
 {
-	if([video_output_view rotation] == ROTATION_0)return;
+	float current_rotation = [self rotation];
 
-	if([video_output_view rotation] == ROTATION_180)
+	if(video_output_view && current_rotation != rotation)
 	{
-		[video_output_view setRotation:ROTATION_0];
+		NSSize size = [video_output_view frame].size;
 
-		[video_output_view setFrame:[video_output_view frame]];
-		[video_output_view display];
+		//rotate view bounds (negative because nsview does it counter-clockwise)
+		[video_output_view setBoundsRotation:-rotation];
+		[video_output_view setNeedsDisplay:YES];
 
-	} else
-	{
-		//set the rotation state
-		[video_output_view setRotation:ROTATION_0];
+		//if view is to be rotated (not just flipped)
+		if(((current_rotation == 0 || current_rotation == 180) && (rotation == 90 || rotation == 270)) ||
+		((current_rotation == 90 || current_rotation == 270) && (rotation == 0 || rotation == 180)))
+		{
+			//swap x-y sizes
+			float temp = size.width;
+			size.width = size.height;
+			size.height = temp;
 
-		//resize the window/view
-		NSSize video_size = [video_output_view frame].size;
-		float temp = video_size.width;
-		video_size.width = video_size.height;
-		video_size.height = temp;
-		[self resizeScreen:video_size];
-
-		//fix the min size
-		[self resetMinSize];
+			//fit window
+			[self resetMinSize:NO];
+			[self resizeScreen:size];
+		}
 	}
 
 	//set the checkmarks
+	rotation = video_output_view ? [self rotation] : -1; //no checks set if video output is nil
 	if([rotation0_item target] == self)
-		[rotation0_item   setState:NSOnState ];
+		if(rotation == 0)[rotation0_item setState:NSOnState];
+		else [rotation0_item setState:NSOffState];
 	if([rotation90_item target] == self)
-		[rotation90_item  setState:NSOffState];
+		if(rotation == 90)[rotation90_item setState:NSOnState];
+		else [rotation90_item setState:NSOffState];
 	if([rotation180_item target] == self)
-		[rotation180_item setState:NSOffState];
+		if(rotation == 180)[rotation180_item setState:NSOnState];
+		else [rotation180_item setState:NSOffState];
 	if([rotation270_item target] == self)
-		[rotation270_item setState:NSOffState];
+		if(rotation == 270)[rotation270_item setState:NSOnState];
+		else [rotation270_item setState:NSOffState];
 }
 
-- (void)setRotation90
-{
-	if([video_output_view rotation] == ROTATION_90)return;
-
-	if([video_output_view rotation] == ROTATION_270)
-	{
-
-		[video_output_view setRotation:ROTATION_90];
-
-		[video_output_view setFrame:[video_output_view frame]];
-		[video_output_view display];
-
-	} else
-	{
-		//set the rotation state
-		[video_output_view setRotation:ROTATION_90];
-
-		//resize the window/screen
-		NSSize video_size = [video_output_view frame].size;
-		float temp = video_size.width;
-		video_size.width = video_size.height;
-		video_size.height = temp;
-		[self resizeScreen:video_size];
-
-		//fix min size
-		[self resetMinSize];
-	}
-
-	//set the checkmarks
-	if([rotation0_item target] == self)
-		[rotation0_item   setState:NSOffState];
-	if([rotation90_item target] == self)
-		[rotation90_item  setState:NSOnState ];
-	if([rotation180_item target] == self)
-		[rotation180_item setState:NSOffState];
-	if([rotation270_item target] == self)
-		[rotation270_item setState:NSOffState];
-}
-
-- (void)setRotation180
-{
-	if([video_output_view rotation] == ROTATION_180)return;
-
-	if([video_output_view rotation] == ROTATION_0)
-	{
-		[video_output_view setRotation:ROTATION_180];
-
-		[video_output_view setFrame:[video_output_view frame]];
-		[video_output_view display];
-
-	} else
-	{
-		//set the rotation state
-		[video_output_view setRotation:ROTATION_180];
-
-		//resize the window/view
-		NSSize video_size = [video_output_view frame].size;
-		float temp = video_size.width;
-		video_size.width = video_size.height;
-		video_size.height = temp;
-		[self resizeScreen:video_size];
-
-		//fix the min size
-		[self resetMinSize];
-	}
-
-	//set the checkmarks
-	if([rotation0_item target] == self)
-		[rotation0_item   setState:NSOffState];
-	if([rotation90_item target] == self)
-		[rotation90_item  setState:NSOffState];
-	if([rotation180_item target] == self)
-		[rotation180_item setState:NSOnState ];
-	if([rotation270_item target] == self)
-		[rotation270_item setState:NSOffState];
-}
-
-- (void)setRotation270
-{
-	if([video_output_view rotation] == ROTATION_270)return;
-
-	if([video_output_view rotation] == ROTATION_90)
-	{
-		[video_output_view setRotation:ROTATION_270];
-
-		[video_output_view setFrame:[video_output_view frame]];
-		[video_output_view display];
-
-	} else
-	{
-
-		//set the rotation state
-		[video_output_view setRotation:ROTATION_270];
-
-		//resize the window/screen
-		NSSize video_size = [video_output_view frame].size;
-		float temp = video_size.width;
-		video_size.width = video_size.height;
-		video_size.height = temp;
-		[self resizeScreen:video_size];
-
-		//fix the min size
-		[self resetMinSize];
-	}
-
-	//set the checkmarks
-	if([rotation0_item target] == self)
-		[rotation0_item   setState:NSOffState];
-	if([rotation90_item target] == self)
-		[rotation90_item  setState:NSOffState];
-	if([rotation180_item target] == self)
-		[rotation180_item setState:NSOffState];
-	if([rotation270_item target] == self)
-		[rotation270_item setState:NSOnState];
-}
+- (void)setRotation0   { [self setRotation:  0]; }
+- (void)setRotation90  { [self setRotation: 90]; }
+- (void)setRotation180 { [self setRotation:180]; }
+- (void)setRotation270 { [self setRotation:270]; }
+- (float)rotation { return video_output_view ? -[video_output_view boundsRotation] : 0; }
 
 - (void)toggleTopBackground0
 {
@@ -944,7 +921,7 @@ NSMenuItem *screenshot_to_file_item;
 }
 
 - (void)saveScreenshot
-{
+{//todo: this should work even if video_output_view doesn't exist
 	BOOL executing = [self executing];
 	[self pause];
 
@@ -961,18 +938,25 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)resetSizeMenuChecks
 {
+	if(video_output_view == nil)
+	{
+		if([resize1x target] == self)[resize1x setState:NSOffState];
+		if([resize2x target] == self)[resize2x setState:NSOffState];
+		if([resize3x target] == self)[resize3x setState:NSOffState];
+		if([resize4x target] == self)[resize4x setState:NSOffState];
+		return;
+	}
+
 	//get the window size
-	NSSize size = [[window contentView] frame].size;
-	size.width -= WINDOW_BORDER_PADDING * 2;
-	size.height -= status_bar_height;
+	NSSize size = [video_output_view frame].size;
 
     //due to rounding errors in the constrain function or whatnot
-    //the size can get off slightly, but we can round for the sake of having menu checkmarks appear correct
+	//the size can get off slightly, but we can round for the sake of having menu checkmarks appear correct
 #define APPRX_EQL(a, b) ((b >= (a - 1)) && (b <= (a + 1)))
 
 	//take rotation into account
-    CGFloat ds_screen_width = (([video_output_view rotation] == ROTATION_0) || ([video_output_view rotation] == ROTATION_180)) ? DS_SCREEN_WIDTH : DS_SCREEN_HEIGHT_COMBINED;
-    CGFloat ds_screen_height = (([video_output_view rotation] == ROTATION_0) || ([video_output_view rotation] == ROTATION_180)) ? DS_SCREEN_HEIGHT_COMBINED : DS_SCREEN_WIDTH;
+	CGFloat ds_screen_width = (([video_output_view boundsRotation] == 0) || ([video_output_view boundsRotation] == -180)) ? DS_SCREEN_WIDTH : DS_SCREEN_HEIGHT_COMBINED;
+	CGFloat ds_screen_height = (([video_output_view boundsRotation] == 0) || ([video_output_view boundsRotation] == -180)) ? DS_SCREEN_HEIGHT_COMBINED : DS_SCREEN_WIDTH;
 
 	if(APPRX_EQL(size.width, ds_screen_width) && APPRX_EQL(size.height, ds_screen_height))
 	{
@@ -1009,26 +993,17 @@ NSMenuItem *screenshot_to_file_item;
 
 - (void)windowDidResize:(NSNotification*)aNotification;
 {
-	//this is called after the window frame resizes
-	//so we must recalculate the video output view
-	NSRect rect = [[window contentView] frame];
-	rect.origin.x = WINDOW_BORDER_PADDING;
-	rect.origin.y = status_bar_height;
-	rect.size.width -= WINDOW_BORDER_PADDING*2;
-	rect.size.height -= status_bar_height;
-	[video_output_view setFrame:rect];
+	if(video_output_view == nil)return;
 
     //set the menu items
 	[self resetSizeMenuChecks];
-
-    //recalculate the status_view
-	rect.origin.y = 0;
-	rect.size.height = status_bar_height;
-	[status_view setFrame:rect];
 }
 
 - (NSSize)windowWillResize:(NSWindow*)sender toSize:(NSSize)proposedFrameSize
 {
+	if(video_output_view == nil)
+		//allow x resize if theres no video output, but not y resize
+		return NSMakeSize(proposedFrameSize.width, [window frameRectForContentRect:NSMakeRect(0, 0, proposedFrameSize.width, [self statusBarHeight])].size.height);
 
 	//constrain proportions
 	if(keep_proportions)
@@ -1039,25 +1014,25 @@ NSMenuItem *screenshot_to_file_item;
 		//this is a simple algorithm to constrain to the smallest axis
 		NSSize constrained_size;
 
-		if([video_output_view rotation] == ROTATION_0 || [video_output_view rotation] == ROTATION_180)
+		if([video_output_view boundsRotation] == 0 || [video_output_view boundsRotation] == -180)
 		{
 			//this is a simple algorithm to constrain to the smallest axis
 
-			constrained_size.width = (content_size.height - status_bar_height) * DS_SCREEN_X_RATIO;
+			constrained_size.width = (content_size.height - [self statusBarHeight]) * DS_SCREEN_X_RATIO;
 			constrained_size.width += WINDOW_BORDER_PADDING*2;
 
 			constrained_size.height = (content_size.width - WINDOW_BORDER_PADDING*2) * DS_SCREEN_Y_RATIO;
-			constrained_size.height += status_bar_height;
+			constrained_size.height += [self statusBarHeight];
 
 		} else
 		{
 			//like above but with opposite ratios
 
-			constrained_size.width = (content_size.height - status_bar_height) * DS_SCREEN_Y_RATIO;
+			constrained_size.width = (content_size.height - [self statusBarHeight]) * DS_SCREEN_Y_RATIO;
 			constrained_size.width += WINDOW_BORDER_PADDING*2;
 
 			constrained_size.height = (content_size.width - WINDOW_BORDER_PADDING*2) * DS_SCREEN_X_RATIO;
-			constrained_size.height += status_bar_height;
+			constrained_size.height += [self statusBarHeight];
 		}
 
 		constrained_size = [window frameRectForContentRect:NSMakeRect(0, 0, constrained_size.width, constrained_size.height)].size;
@@ -1078,26 +1053,32 @@ NSMenuItem *screenshot_to_file_item;
 - (void)setVolume:(int)volume
 {
 	[super setVolume:volume];
-	[mute_item setState:NSOffState];
+	
+	int i;
+	for(i = 0; i < 10; i++)
+	if([volume_item[i] target] == self)
+		if(volume == (i+1)*10)
+			[volume_item[i] setState:NSOnState];
+		else
+			[volume_item[i] setState:NSOffState];
 }
 
 - (void)setVolumeFromMenu:(id)sender
 {
 	int i;
 	for(i = 0; i < 10; i++)
+	if(sender == volume_item[i])
 	{
-		if(sender == volume_item[i])
-		{
-			[volume_item[i] setState:NSOnState];
-			[self setVolume:(i+1)*10];
-		} else
-			[volume_item[i] setState:NSOffState];
+		[self disableMute]; //unmute if needed
+		[self setVolume:(i+1)*10];
+		break;
 	}
 }
 
 - (void)enableMute
 {
 	[super enableMute];
+
 	if([mute_item target] == self)
 		[mute_item setState:NSOnState];
 }
@@ -1105,6 +1086,7 @@ NSMenuItem *screenshot_to_file_item;
 - (void)disableMute
 {
 	[super disableMute];
+	
 	if([mute_item target] == self)
 		[mute_item setState:NSOffState];
 }
@@ -1124,8 +1106,9 @@ NSMenuItem *screenshot_to_file_item;
 		[loadSlot_item[i] setTarget:self];
 
 		if([self saveStateExists:i] == YES)
+		{
 			[saveSlot_item[i] setState:NSOnState];
-		else
+		} else
 			[saveSlot_item[i] setState:NSOffState];
 	}
 
@@ -1151,20 +1134,18 @@ NSMenuItem *screenshot_to_file_item;
 		[pause_item setState:NSOffState];
 	}
 
-	for(i = 0; i < MAX_FRAME_SKIP; i++)
-	{
-		[frame_skip_item[i] setTarget:self];
-		if([self frameSkip] == i)
-			[frame_skip_item[i] setState:NSOnState];
-		else
-			[frame_skip_item[i] setState:NSOffState];
-	}
 	[frame_skip_auto_item setTarget:self];
+	for(i = 0; i < MAX_FRAME_SKIP; i++)[frame_skip_item[i] setTarget:self];
+	[self setFrameSkip:[self frameSkip]]; //set the menu checkmarks correctly
 
-	if([self frameSkip] < 0)
-		[frame_skip_auto_item setState:NSOnState];
-	else
-		[frame_skip_auto_item setState:NSOffState];
+	[speed_limit_25_item setTarget:self];
+	[speed_limit_50_item setTarget:self];
+	[speed_limit_75_item setTarget:self];
+	[speed_limit_100_item setTarget:self];
+	[speed_limit_200_item setTarget:self];
+	[speed_limit_none_item setTarget:self];
+	[speed_limit_custom_item setTarget:self];
+	[self setSpeedLimit:[self speedLimit]]; //this will set the checks correctly
 
 	//VIEW menu
 
@@ -1228,31 +1209,8 @@ NSMenuItem *screenshot_to_file_item;
 		[subBG3_item setState:NSOnState];
 	else
 		[subBG3_item setState:NSOffState];
-	if([video_output_view rotation] == ROTATION_0)
-	{
-		[rotation0_item   setState:NSOnState ];
-		[rotation90_item  setState:NSOffState];
-		[rotation180_item setState:NSOffState];
-		[rotation270_item setState:NSOffState];
-	} else if([video_output_view rotation] == ROTATION_90)
-	{
-		[rotation0_item   setState:NSOffState];
-		[rotation90_item  setState:NSOnState ];
-		[rotation180_item setState:NSOffState];
-		[rotation270_item setState:NSOffState];
-	} else if([video_output_view rotation] == ROTATION_180)
-	{
-		[rotation0_item   setState:NSOffState];
-		[rotation90_item  setState:NSOffState];
-		[rotation180_item setState:NSOnState ];
-		[rotation270_item setState:NSOffState];
-	} else if([video_output_view rotation] == ROTATION_270)
-	{
-		[rotation0_item   setState:NSOffState];
-		[rotation90_item  setState:NSOffState];
-		[rotation180_item setState:NSOffState];
-		[rotation270_item setState:NSOnState ];
-	} else messageDialog(NSLocalizedString(@"Error", nil), @"Corrupt rotation variable");
+
+	[self setRotation:[self rotation]];
 
 	//SOUND Menu
 	int volume;
@@ -1285,13 +1243,13 @@ NSMenuItem *screenshot_to_file_item;
 
 - (BOOL)validateMenuItem:(NSMenuItem*)item
 {
-	//This function is called automaticlly by Cocoa
+	//This function is called automatically by Cocoa
 	//when it needs to know if a menu item should be greyed out
 
 	int i;
 
 	if([self ROMLoaded] == NO)
-	{ //if no rom is loaded, various options are disables
+	{ //if no rom is loaded, various options are disabled
 		if(item == close_rom_item)return NO;
 		if(item == rom_info_item)return NO;
 		if(item == save_state_as_item)return NO;
@@ -1308,13 +1266,31 @@ NSMenuItem *screenshot_to_file_item;
 
 		if(item == screenshot_to_file_item)return NO;
 	}
-	
-	else
-	
-	for(i = 0; i < MAX_SLOTS; i++)
-		if(item == loadSlot_item[i])
-			if([self saveStateExists:i]==NO)return NO;
 
+	else
+		for(i = 0; i < MAX_SLOTS; i++)
+			if(item == loadSlot_item[i])
+				if([self saveStateExists:i]==NO)return NO;
+
+	if(video_output_view == nil)
+	{
+		if(item == resize1x)return NO;
+		if(item == resize2x)return NO;
+		if(item == resize3x)return NO;
+		if(item == resize4x)return NO;
+		if(item == constrain_item)return NO;
+		if(item == rotation0_item)return NO;
+		if(item == rotation90_item)return NO;
+		if(item == rotation180_item)return NO;
+		if(item == rotation270_item)return NO;
+		if(item == screenshot_to_file_item)return NO;
+	}
+	
+	if([self hasSound] == NO)
+	{
+		if(item == mute_item)return NO;
+		for(i = 0; i < 10; i++)if(item == volume_item[i])return NO;
+	}
 
 	return YES;
 }
