@@ -17,20 +17,29 @@
 	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#define PREF_EXECUTE_UPON_LOAD @"Execute Upon Load"
-#define PREF_FLASH_FILE @"FlashFile"
-#define PREF_KEY_A @"KeyA"
-#define PREF_KEY_B @"KeyB"
-#define PREF_KEY_X @"KeyX"
-#define PREF_KEY_Y @"KeyY"
-#define PREF_KEY_L @"KeyL"
-#define PREF_KEY_R @"KeyR"
-#define PREF_KEY_UP  @"KeyUp"
-#define PREF_KEY_DOWN @"KeyDown"
-#define PREF_KEY_LEFT @"KeyLeft"
-#define PREF_KEY_RIGHT @"KeyRight"
-#define PREF_KEY_START @"KeyStart"
-#define PREF_KEY_SELECT @"KeySelect"
+#import <Cocoa/Cocoa.h>
 
+#define PREF_EXECUTE_UPON_LOAD @"Execute Upon Load"
+
+#define PREF_AFTER_LAUNCHED @"When Launching, Load"
+#define PREF_AFTER_LAUNCHED_OPTION_NOTHING @"Load Nothing"
+#define PREF_AFTER_LAUNCHED_OPTION_LAST_ROM @"Load Last ROM"
+
+#define PREF_FLASH_FILE @"Flash File"
+
+#define PREF_KEY_A @"A Button"
+#define PREF_KEY_B @"B Button"
+#define PREF_KEY_X @"X Button"
+#define PREF_KEY_Y @"Y Button"
+#define PREF_KEY_L @"L Button"
+#define PREF_KEY_R @"R Button"
+#define PREF_KEY_UP  @"Up Button"
+#define PREF_KEY_DOWN @"Down Button"
+#define PREF_KEY_LEFT @"Left Button"
+#define PREF_KEY_RIGHT @"Right Button"
+#define PREF_KEY_START @"Start Button"
+#define PREF_KEY_SELECT @"Select Button"
 
 void setAppDefaults(); //this is defined in preferences.m and should be called at app launch
+
+NSView *createPreferencesView(NSString *helpinfo, NSDictionary *options, id delegate); //utility func for creating a preference panel with a set of options
