@@ -29,15 +29,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-//#include <libgen.h>
 #include <unistd.h>
 
 // Localization
 #include <libintl.h>
 #define _(String) gettext (String)
 
-//#include <SDL/SDL.h>
-// more portable
 #include "SDL.h"
 
 // fix gtk-glade on windows with no configure
@@ -94,10 +91,8 @@ typedef union _callback_arg{
 #include "../ctrlssdl.h"
 #include "../types.h"
 #include "../saves.h"
-#include "../gl_vertex.h"
 #include "../render3D.h"
 #include "desmume.h"
-#include "gdk_gl.h"
 
 // autoconnect with strings as user_data
 
@@ -106,13 +101,12 @@ glade_xml_signal_autoconnect_StringObject (GladeXML *self);
 
 //---
 
-int Frameskip;
-u16 Keypad_Temp[NB_KEYS];
+extern int Frameskip;
 
-/* main.c */
-GtkWidget * pWindow;
-GtkWidget * pDrawingArea, * pDrawingArea2;
-GladeXML  * xml, * xml_tools;
+/* main.cpp */
+extern GtkWidget * pWindow;
+extern GtkWidget * pDrawingArea, * pDrawingArea2;
+extern GladeXML  * xml, * xml_tools;
 
 typedef void (*VoidFunPtr)();
 void notify_Tools();
@@ -120,28 +114,18 @@ void register_Tool(VoidFunPtr fun);
 void unregister_Tool(VoidFunPtr fun);
 gchar * get_ui_file (const char *filename);
 
-/* callbacks.c */
+/* callbacks.cpp */
 void enable_rom_features();
 void resize (float Size1, float Size2);
 void rotate(float angle);
-gboolean ScreenInvert;
-gboolean ScreenGap;
-gboolean ScreenRight;
+extern gboolean ScreenInvert;
 
-/* callbacks_IO.c */
-float ScreenCoeff_Size[2];
-float ScreenRotate;
+/* callbacks_IO.cpp */
+extern float ScreenCoeff_Size[2];
+extern float ScreenRotate;
 
 void black_screen ();
 void edit_controls();
 void init_joy_labels();
 
-/* printscreen.c */
-int WriteBMP(const char *filename,u16 *bmp);
-
-/* keyvalnames.c   -see <gdk/gdkkeysyms.h>- */
-char * KEYVAL_NAMES[0x10000];
-char * KEYNAME(int k);
-void init_keyvals();
-
 #endif /* __GLOBALS_H__ */
