@@ -17,14 +17,12 @@
     along with DeSmuME; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
-#ifdef EXPERIMENTAL_WIFI
-
 #ifndef WIFI_H
 #define WIFI_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include "types.h"
+
+#ifdef EXPERIMENTAL_WIFI
 
 /* standardize socket interface for linux and windows */
 #ifdef WIN32
@@ -32,6 +30,10 @@ extern "C" {
 	#define socket_t 	SOCKET
 	#define sockaddr_t  SOCKADDR
 #else
+	#include <unistd.h>
+	#include <stdlib.h>
+	#include <string.h>
+	#include <arpa/inet.h>
 	#include <sys/socket.h>
 	#define socket_t 	int
 	#define sockaddr_t  struct sockaddr
@@ -42,8 +44,7 @@ extern "C" {
 #define BASEPORT        7000    		/* channel 1: 7000 ... channel 13: 7012 */
 										/* FIXME: make it configureable */
 
-#include "types.h"
-
+#define		REG_WIFI_ID					0x000
 #define     REG_WIFI_MODE       		0x004
 #define     REG_WIFI_WEP        		0x006
 #define     REG_WIFI_IF     			0x010
@@ -54,10 +55,11 @@ extern "C" {
 #define     REG_WIFI_BSS0       		0x020
 #define     REG_WIFI_BSS1       		0x022
 #define     REG_WIFI_BSS2       		0x024
-#define     REG_WIFI_AID        		0x028
-#define     REG_WIFI_AIDCPY        		0x02A
+#define     REG_WIFI_AID_LOW       		0x028
+#define     REG_WIFI_AID_HIGH      		0x02A
 #define     REG_WIFI_RETRYLIMIT 		0x02C
 #define		REG_WIFI_WEPCNT				0x032
+#define		REG_WIFI_POWER_US			0x036
 #define     REG_WIFI_POWERSTATE 		0x03C
 #define     REG_WIFI_FORCEPS    		0x040
 #define     REG_WIFI_RANDOM     		0x044
@@ -67,7 +69,10 @@ extern "C" {
 #define     REG_WIFI_WRITECSRLATCH      0x056
 #define     REG_WIFI_CIRCBUFRADR        0x058
 #define     REG_WIFI_RXREADCSR          0x05A
+#define     REG_WIFI_RXBUF_COUNT		0x05C
 #define     REG_WIFI_CIRCBUFREAD        0x060
+#define     REG_WIFI_CIRCBUFRD_END      0x062
+#define     REG_WIFI_CIRCBUFRD_SKIP     0x064
 #define     REG_WIFI_CIRCBUFWADR        0x068
 #define     REG_WIFI_CIRCBUFWRITE       0x070
 #define     REG_WIFI_CIRCBUFWR_END      0x074
@@ -85,6 +90,7 @@ extern "C" {
 #define     REG_WIFI_RXFILTER           0x0D0
 #define     REG_WIFI_USCOUNTERCNT       0x0E8
 #define     REG_WIFI_USCOMPARECNT       0x0EA
+#define		REG_WIFI_EXTRACOUNTCNT		0x0EE
 #define     REG_WIFI_USCOMPARE0         0x0F0
 #define     REG_WIFI_USCOMPARE1         0x0F2
 #define     REG_WIFI_USCOMPARE2         0x0F4
@@ -93,6 +99,7 @@ extern "C" {
 #define     REG_WIFI_USCOUNTER1         0x0FA
 #define     REG_WIFI_USCOUNTER2         0x0FC
 #define     REG_WIFI_USCOUNTER3         0x0FE
+#define		REG_WIFI_EXTRACOUNT			0x118
 #define     REG_WIFI_BBSIOCNT           0x158
 #define     REG_WIFI_BBSIOWRITE         0x15A
 #define     REG_WIFI_BBSIOREAD          0x15C
@@ -101,6 +108,11 @@ extern "C" {
 #define     REG_WIFI_RFIODATA1  		0x17E
 #define     REG_WIFI_RFIOBSY    		0x180
 #define     REG_WIFI_RFIOCNT    		0x184
+#define		REG_WIFI_IF_SET				0x21C
+
+/* WIFI misc constants */
+#define		WIFI_CHIPID					0x1440		/* emulates "old" wifi chip, new is 0xC340 */
+#define		REG_PWRCNT					0x04000304	
 
 /* Referenced as RF_ in dswifi: rffilter_t */
 /* based on the documentation for the RF2958 chip of RF Micro Devices */
@@ -390,13 +402,17 @@ typedef struct
 		u8	 bytes[6] ;
 	} bss ;
 	u16 aid ;
+	u16 pid ; /* player ID or aid_low */
 	u16 retryLimit ;
 
 	/* timing */
+	BOOL crystalEnabled ;
 	u64 usec ;
 	BOOL usecEnable ;
 	u64 ucmp ;
 	BOOL ucmpEnable ;
+	u16 eCount ;
+	BOOL eCountEnable ;
 
 	/* subchips */
     rffilter_t 	RF ;
@@ -415,10 +431,13 @@ typedef struct
 	u16         RXHWWriteCursor ;
 	u16         RXReadCursor ;
 	u16         RXUnits ;
+	u16			RXBufCount ;
 	u16         CircBufReadAddress ;
 	u16         CircBufWriteAddress ;
-	u16         CircBufEnd ;
-	u16         CircBufSkip ;
+	u16         CircBufRdEnd ;
+	u16         CircBufRdSkip ;
+	u16         CircBufWrEnd ;
+	u16         CircBufWrSkip ;
 
 	/* others */
 	u16			randomSeed ;
@@ -450,19 +469,44 @@ u16  WIFI_read16(wifimac_t *wifi,u32 address) ;
 /* wifimac timing */
 void WIFI_usTrigger(wifimac_t *wifi) ;
 
+#endif
+
+/* DS WFC profile data documented here : */
+/* http://dsdev.bigredpimp.com/2006/07/31/aoss-wfc-profile-data/ */
+/* Note : we use bytes to avoid endianness issues */
+typedef struct _FW_WFCProfile
+{
+	char SSID[32];
+	char SSID_WEP64[32];
+	char WEPKEY_PART1[16];
+	char WEPKEY_PART2[16];
+	char WEPKEY_PART3[16];
+	char WEPKEY_PART4[16];
+	u8 IP_ADDRESS[4];
+	u8 GATEWAY[4];
+	u8 PRIM_DNS[4];
+	u8 SEC_DNS[4];
+	u8 SUBNET_MASK;
+	u8 WEP64_KEY_AOSS[20];
+	u8 UNK1;
+	u8 WEP_MODE;
+	u8 STATUS;
+	u8 UNK2[7];
+	u8 UNK3;
+	u8 UNK4[14];
+	u8 CRC16[2];
+
+} FW_WFCProfile;
+
 /* wifi data to be stored in firmware, when no firmware image was loaded */
-extern u8 FW_Mac[6];
-extern u8 FW_WIFIInit[32] ;
-extern u8 FW_BBInit[105] ;
-extern u8 FW_RFInit[36] ;
-extern u8 FW_RFChannel[6*14] ;
-extern u8 FW_BBChannel[14] ;
-extern u8 FW_WFCProfile[0xC0] ;
-
-#ifdef __cplusplus
-}
-#endif
+extern const u8 FW_Mac[6];
+extern const u8 FW_WIFIInit[32] ;
+extern const u8 FW_BBInit[105] ;
+extern const u8 FW_RFInit[36] ;
+extern const u8 FW_RFChannel[6*14] ;
+extern const u8 FW_BBChannel[14] ;
+extern FW_WFCProfile FW_WFCProfile1;
+extern FW_WFCProfile FW_WFCProfile2;
+extern FW_WFCProfile FW_WFCProfile3;
 
 #endif
-
-#endif /* #ifdef EXPERIMENTAL_WIFI */
