@@ -21,7 +21,8 @@
 
 #ifndef IO_REG_H
 #define IO_REG_H
+#include "../common.h"
 
-BOOL CALLBACK IoregView_Proc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
+extern BOOL CALLBACK IoregView_Proc (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
 
 #endif 
